//arduino 与 stm32 通过spi通信   arduino 作为主机  stm32作为从机     arduino 与 蓝牙 通过串口通信  arduino 与 nrf 通过spi通信
#include <SPI.h>

#define QUAD_CS   D8                       // 给digitalWrite()函数使用，所以是D8引脚  片选信号 可以随便指定
#define NRF_CS   D10

#define CMD_NONE                  0x00
#define CMD_SET_CH4               0x01           //对应 RAW PITCH YAW THROTTLE四个通道
#define CMD_ARM_IT_SWITCH         0x02           //使用总开关控制加锁、解锁 
#define CMD_HEAD_IT               0x03           //有头模式 无头模式   
#define CMD_ACC_CLI               0x04           //校准 
#define CMD_AUTO_FLY_UP           0x05           //自动起飞
#define CMD_AUTO_LANDING          0x07           //自动降落
#define CMD_PID_SET               0x08
#define CMD_ARM_IT_STICK          0x09           //使用摇杆控制加锁、解锁


#define STEP_WAIT_AA      1
#define STEP_WAIT_BB      2
#define STEP_WAIT_LEN     3
#define STEP_WAIT_DATA    4
#define STEP_WAIT_BCC     5
#define DATA_TX_LEN       10        //发送数据的数据位长度
#define DATA_LENGTH       16        //接收数据的数据位长度
#define TOTAL_LENGTH      21        //接收数据的总长度

enum State {
  WAIT_AA = 0, WAIT_BB, WAIT_FUNC, WAIT_LEN, WAIT_DATA, WAIT_CRC
} ;
int state = 0;

int data_len_cnt = 0;  

uint8_t rxBuffer[100];                //用于存储spi收到的数据
uint8_t rxData[TOTAL_LENGTH] = {};    //用于存储从rxBuffer整理后的数据
uint8_t Data_to_Rc[TOTAL_LENGTH - 2];     //用于发送给遥控器的数据  因为添加了2字节的软件版本数据，但遥控器不需要
uint8_t SoftVersion[2] = {};              //用于存储表示软件版本的2byte数据

typedef union _byteToint {
  int16_t twobyte;
  uint8_t Twobyte[2];
} _byteToint;

void Data_Parse(uint8_t data)
{
  static int data_len_cnt = 0;
  switch (state)
  {
    case WAIT_AA:
      if (data == 0xAA)
      {
        rxData[0] = data;
        state = WAIT_BB;
      }
      else
        state = WAIT_AA;
      break;
    case WAIT_BB:
      if (data == 0xBB)
      {
        rxData[1] = data;
        state = WAIT_FUNC;
      }
      else
        state = WAIT_BB;
      break;
    case WAIT_FUNC:
      if (data == 0x00)
      {
        rxData[2] = data;
        state = WAIT_LEN;
      }
      else
        state = WAIT_AA;
      break;

    case WAIT_LEN:
      if (data == DATA_LENGTH)
      {
        rxData[3] = DATA_LENGTH;
        state = WAIT_DATA;
      }
      else
        state = WAIT_AA;
      break;
    case WAIT_DATA:
      rxData[4 + data_len_cnt++] = data;
      if (data_len_cnt > DATA_LENGTH - 1)
      {
        data_len_cnt = 0;
        state = WAIT_CRC;
      }
      else
        state = WAIT_DATA;
      break;
    case WAIT_CRC:
      rxData[TOTAL_LENGTH - 1] = data;

      //添加了2byte的软件版本，但不需要发给遥控器
      memcpy(Data_to_Rc, rxData, TOTAL_LENGTH-3);
      Data_to_Rc[TOTAL_LENGTH - 3] = rxData[TOTAL_LENGTH - 1];
      SoftVersion[1] = rxData[TOTAL_LENGTH - 3];
      SoftVersion[0] = rxData[TOTAL_LENGTH - 2];       
      state = WAIT_AA;
      break;
  }
}


void QuadBegin() {
  SPI.begin();
  pinMode(QUAD_CS, OUTPUT);
  digitalWrite(QUAD_CS, HIGH);  
}

uint8_t QuadWrite(uint8_t cmd1, uint8_t cmd2, uint16_t *data) {  
  uint8_t rxResult = 0;
  uint8_t txData[14];
  txData[0] = 0xAA;
  txData[1] = 0XBB;
  txData[2] = DATA_TX_LEN;
  txData[13] = DATA_TX_LEN;
  txData[3] = cmd1;
  txData[4] = cmd2;
 
  memcpy(txData + 5, (uint8_t *)data, 8);
    
  for (uint8_t i = 3; i < 13; i++)
    txData[13] ^= txData[i];
    
  digitalWrite(QUAD_CS, LOW);
  for (uint8_t i = 0; i < 14; i++) 
  {    
   rxBuffer[i] = SPI.transfer(txData[i]);
   //Serial.print(txData[i]);
   Data_Parse(rxBuffer[i]);
  }
  digitalWrite(QUAD_CS, HIGH);  
}

void Get_SoftDeviceVersion(void)
{
  _byteToint Version;
  Version.Twobyte[1] = SoftVersion[1];
  Version.Twobyte[0] = SoftVersion[0];

  Serial.print("四轴软件版本号为： ");
  Serial.println(Version.twobyte/10.0);   
}

